@@ -1,8 +1,8 @@
-/*
+/** -*- mode: objc -*-
  Project: DataBasin
  DBSobject.h
  
- Copyright (C) 2010-2015 Free Software Foundation
+ Copyright (C) 2010-2016 Free Software Foundation
  
  Author: Riccardo Mottola
  
@@ -116,4 +116,8 @@
  */
 - (void)loadValuesForFields:(NSArray *)namesArray;
 
+/** <p>Stores by updating the value of the fields passed in the array.</p>
+ */
+- (void)storeValuesForFields:(NSArray *)namesArray;
+
 @end
