@@ -1,7 +1,7 @@
 /*
   Project: DataBasin
 
-  Copyright (C) 2008-2016 Free Software Foundation
+  Copyright (C) 2008-2018 Free Software Foundation
 
   Author: Riccardo Mottola
 
@@ -35,8 +35,11 @@
 #endif
 #endif
 
-#define MAX_SOQL_SIZE 9000
+/* some Salesforce limits */
+#define MAX_SOQL_LENGTH 20000
+#define MAX_SOQL WHERE_LENGTH 4000
 #define MAX_BATCH_SIZE 200
+#define RETRIEVE_BATCH_SIZE 2000
 
 #define CLIENT_NAME @"DataBasin"
 
@@ -59,12 +62,17 @@
   NSArray  *sObjectList;
   /* list of all object names, custom and not */
   NSMutableArray  *sObjectNamesList;
+  /* Map of described objects, for which details are present */
+  NSMutableDictionary *sObjectDetailsDict;
 
   /** create, update, upsert batch size */
   unsigned upBatchSize;
 
   /** query batch size */
   unsigned downBatchSize;
+  
+  /** actually used SOQL length, must be inferior to MAX_SOQL_LENGTH */
+  unsigned maxSOQLLength;
 
   /** Timeout in seconds, for generic methods */
   unsigned standardTimeoutSec;
@@ -76,6 +84,9 @@
   NSUInteger *busyCount;
   NSRecursiveLock *lockBusy;
 
+  /** describe each object in a query to get field types */
+  BOOL enableFieldTypesDescribeForQuery;
+
   /** return positive results */
   BOOL returnSuccessResults;
 
@@ -95,6 +106,8 @@
 - (void)setUpBatchSize:(unsigned)size;
 - (unsigned)downBatchSize;
 - (void)setDownBatchSize:(unsigned)size;
+- (unsigned)maxSOQLLength;
+- (void)setMaxSOQLLength:(unsigned)size;
 - (NSMutableArray *)queryFull :(NSString *)queryString queryAll:(BOOL)all progressMonitor:(id<DBProgressProtocol>)p;
 - (NSString *)query :(NSString *)queryString queryAll:(BOOL)all toArray:(NSMutableArray *)objects progressMonitor:(id<DBProgressProtocol>)p;
 - (NSString *)queryMore :(NSString *)locator toArray:(NSMutableArray *)objects;
@@ -103,12 +116,24 @@
 - (NSMutableArray *)update :(NSString *)objectName fromArray:(NSMutableArray *)objects progressMonitor:(id<DBProgressProtocol>)p;
 - (NSMutableArray *)delete :(NSArray *)array progressMonitor:(id<DBProgressProtocol>)p;
 
+- (NSMutableArray *)retrieveWithQuery:(NSString *)queryString andObjects:(NSArray *)objectList;
+- (NSMutableArray *)retrieveFields:(NSArray *)fieldList ofObject:(NSString *)objectType fromArray:(NSArray *)objectList;
+
+
+- (NSMutableArray *)getUpdated :(NSString *)objectType :(NSDate *)startDate :(NSDate *)endDate;
+- (NSMutableArray *)getDeleted :(NSString *)objectType :(NSDate *)startDate :(NSDate *)endDate;
+
 - (NSArray *)describeGlobal;
 - (NSArray *)sObjects;
 - (NSArray *)sObjectNames;
 - (void)updateObjects;
 - (DBSObject *)describeSObject: (NSString *)objectType;
 - (NSString *)identifyObjectById:(NSString *)sfId;
+- (void)flushObjectDetails;
+- (BOOL)enableFieldTypesDescribeForQuery;
+- (void)setEnableFieldTypesDescribeForQuery:(BOOL)flag;
+- (NSMutableDictionary *)sObjectDetailsDict;
+- (void)setSObjectDetailsDict:(NSMutableDictionary *)md;
 
 - (NSString *) sessionId;
 - (void) setSessionId:(NSString *)session;
@@ -127,4 +152,45 @@
 
 @end
 
+@interface DBSoap (Selecting)
 
+- (NSString *)_query :(NSString *)queryString queryAll:(BOOL)all toArray:(NSMutableArray *)objects declaredSize:(NSUInteger *)ds progressMonitor:(id<DBProgressProtocol>)p;
+
+- (NSString *)_queryMore :(NSString *)locator toArray:(NSMutableArray *)objects;
+
+- (NSMutableArray *)_queryFull :(NSString *)queryString queryAll:(BOOL)all progressMonitor:(id<DBProgressProtocol>)p;
+
+- (void)_queryIdentify :(NSString *)queryString with: (NSArray *)identifiers queryAll:(BOOL)all fromArray:(NSArray *)fromArray toArray:(NSMutableArray *)outArray withBatchSize:(int)batchSize progressMonitor:(id<DBProgressProtocol>)p;
+
+- (NSMutableArray *)_retrieveFields:(NSArray *)fieldList ofObject:(NSString*)objectType fromObjects:(NSArray *)objectList;
+
+
+@end
+
+
+@interface DBSoap (Updating)
+
+- (NSMutableArray *)_update :(NSString *)objectName fromArray:(NSMutableArray *)objects progressMonitor:(id<DBProgressProtocol>)p;
+
+@end
+
+
+@interface DBSoap (Creating)
+
+- (NSMutableArray *)_create :(NSString *)objectName fromArray:(NSMutableArray *)objects progressMonitor:(id<DBProgressProtocol>)p;
+
+@end
+
+
+@interface DBSoap (Deleting)
+
+- (NSMutableArray *)_delete :(NSArray *)array progressMonitor:(id<DBProgressProtocol>)p;
+
+@end
+
+@interface DBSoap (PrivateMethods)
+- (DBSObject *)_describeSObject: (NSString *)objectType;
+- (NSArray *)_describeGlobal;
+- (id)adjustFormatForField:(NSString *)key forValue:(id)value inObject:(DBSObject *)sObj;
+- (void)extractQueryRecords:(NSArray *)records toObjects:(NSMutableArray *)objects;
+@end
